@@ -12,8 +12,8 @@
 
 """Here is defined the Index class."""
 
+from __future__ import print_function
 import sys
-from bisect import bisect_left, bisect_right
 from time import time, clock
 import os
 import os.path
@@ -37,6 +37,9 @@ from tables.group import Group
 from tables.path import join_path
 from tables.exceptions import PerformanceWarning
 from tables.utils import is_idx, idx2long, lazyattr
+from tables.utilsextension import (nan_aware_gt, nan_aware_ge,
+                                   nan_aware_lt, nan_aware_le,
+                                   bisect_left, bisect_right)
 from tables.lrucacheextension import ObjectCache
 
 from tables._past import previous_api, previous_api_property
@@ -58,8 +61,9 @@ if profile:
 
 
 # The default method for sorting
-defsort = "quicksort"
-# defsort = "mergesort"
+# defsort = "quicksort"
+# Changing to mergesort to fix #441
+defsort = "mergesort"
 
 # Default policy for automatically updating indexes after a table
 # append operation, or automatically reindexing after an
@@ -155,7 +159,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
     _c_classId = previous_api_property('_c_classid')
 
-    # <properties>
     kind = property(
         lambda self: {1: 'ultralight', 2: 'light',
                       4: 'medium', 8: 'full'}[self.indsize],
@@ -163,7 +166,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         "The kind of this index.")
 
     filters = property(
-        lambda self: self._v_filters, None, None, 
+        lambda self: self._v_filters, None, None,
         """Filter properties for this index - see Filters in
         :ref:`FiltersClassDescr`.""")
 
@@ -193,12 +196,13 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         """)
 
     def _getcolumn(self):
-        tablepath, columnpath = _table_column_pathname_of_index(self._v_pathname)
+        tablepath, columnpath = _table_column_pathname_of_index(
+            self._v_pathname)
         table = self._v_file._get_node(tablepath)
         column = table.cols._g_col(columnpath)
         return column
 
-    column = property(_getcolumn, None, None, 
+    column = property(_getcolumn, None, None,
         """The Column (see :ref:`ColumnClassDescr`) instance for the indexed
         column.""")
 
@@ -209,7 +213,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         return table
 
     table = property(_gettable, None, None,
-        "Accessor for the `Table` object of this index.")
+                     "Accessor for the `Table` object of this index.")
 
     nblockssuperblock = property(
         lambda self: self.superblocksize // self.blocksize, None, None,
@@ -326,7 +330,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             pass
         return lbucket
 
-    # </properties>
     def __init__(self, parentnode, name,
                  atom=None, title="",
                  kind=None,
@@ -450,8 +453,8 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
                 nboundsLR = 0  # correction for -1 bounds
             nboundsLR += 2  # bounds + begin + end
             # All bounds values (+begin + end) are at the end of sortedLR
-            self.bebounds = self.sortedLR[nelementsSLR:nelementsSLR +
-                                                                nboundsLR]
+            self.bebounds = self.sortedLR[
+                nelementsSLR:nelementsSLR + nboundsLR]
             return
 
         # The index is new. Initialize the values
@@ -474,7 +477,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         (self.superblocksize, self.blocksize,
          self.slicesize, self.chunksize) = self.blocksizes
         if debug:
-            print "blocksizes:", self.blocksizes
+            print("blocksizes:", self.blocksizes)
         # Compute the reduction level
         self.reduction = get_reduction_level(
             self.indsize, self.optlevel, self.slicesize, self.chunksize)
@@ -655,7 +658,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         return idx
 
     def append(self, xarr, update=False):
-        """Append the array to the index objects"""
+        """Append the array to the index objects."""
 
         if profile:
             tref = time()
@@ -720,7 +723,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             show_stats("Exiting append", tref)
 
     def append_last_row(self, xarr, update=False):
-        """Append the array to the last row index objects"""
+        """Append the array to the last row index objects."""
 
         if profile:
             tref = time()
@@ -795,7 +798,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         optmedian, optstarts, optstops, optfull = opts
 
         if debug:
-            print "optvalues:", opts
+            print("optvalues:", opts)
 
         self.create_temp2()
         # Start the optimization process
@@ -956,7 +959,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if self.verbose:
             t = round(time() - t1, 4)
             c = round(clock() - c1, 4)
-            print "time: %s. clock: %s" % (t, c)
+            print("time: %s. clock: %s" % (t, c))
 
     def swap(self, what, mode=None):
         """Swap chunks or slices using a certain bounds reference."""
@@ -985,7 +988,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         if self.verbose:
             t = round(time() - t1, 4)
             c = round(clock() - c1, 4)
-            print "time: %s. clock: %s" % (t, c)
+            print("time: %s. clock: %s" % (t, c))
         # Check that entropy is actually decreasing
         if what == "chunks" and self.last_tover > 0. and self.last_nover > 0:
             tover_var = (self.last_tover - tover) / self.last_tover
@@ -1097,7 +1100,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         """Copy the data and delete the temporaries for sorting purposes."""
 
         if self.verbose:
-            print "Copying temporary data..."
+            print("Copying temporary data...")
         # tmp -> index
         reduction = self.reduction
         cs = self.chunksize // reduction
@@ -1144,7 +1147,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         self.indicesLR.attrs.nelements = self.nelementsILR
 
         if self.verbose:
-            print "Deleting temporaries..."
+            print("Deleting temporaries...")
         self.tmp = None
         self.tmpfile.close()
         os.remove(self.tmpfilename)
@@ -1452,7 +1455,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
                 # so skip the reordering of this superblock
                 # (too expensive for such a little improvement)
                 if self.verbose:
-                    print "skipping reordering of superblock ->", sblock
+                    print("skipping reordering of superblock ->", sblock)
                 continue
             ns = sblock * nss2
             # Swap sorted and indices slices following the new order
@@ -1499,7 +1502,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         # This method will only works under the assumtion that item
         # *is to be found* in the nslice.
-        assert limits[0] < item <= limits[1]
+        assert nan_aware_lt(limits[0], item) and nan_aware_le(item, limits[1])
         cs = self.chunksize
         ss = self.slicesize
         nelementsLR = self.nelementsILR
@@ -1610,9 +1613,9 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             if erange > 0:
                 toverlap = soverlap / erange
         if verbose and message != "init":
-            print "toverlap (%s):" % message, toverlap
-            print "multiplicity:\n", multiplicity, multiplicity.sum()
-            print "overlaps:\n", overlaps, overlaps.sum()
+            print("toverlap (%s):" % message, toverlap)
+            print("multiplicity:\n", multiplicity, multiplicity.sum())
+            print("overlaps:\n", overlaps, overlaps.sum())
         noverlaps = overlaps.sum()
         # For full indexes, set the 'is_csi' flag
         if self.indsize == 8 and self._v_file._iswritable():
@@ -1674,8 +1677,8 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
             if erange > 0:
                 toverlap = soverlap / erange
         if verbose:
-            print "overlaps (%s):" % message, noverlaps, toverlap
-            print multiplicity
+            print("overlaps (%s):" % message, noverlaps, toverlap)
+            print(multiplicity)
         # For full indexes, set the 'is_csi' flag
         if self.indsize == 8 and self._v_file._iswritable():
             self._v_attrs.is_csi = (noverlaps == 0)
@@ -1827,7 +1830,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         self.dirtycache = False
 
     def search(self, item):
-        """Do a binary search in this index for an item"""
+        """Do a binary search in this index for an item."""
 
         if profile:
             tref = time()
@@ -1952,8 +1955,8 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
 
         nchunk = -1
         # Lookup for item1
-        if item1 > b0:
-            if item1 <= b1:
+        if nan_aware_gt(item1, b0):
+            if nan_aware_le(item1, b1):
                 # Search the appropriate chunk in bounds cache
                 nchunk = bisect_left(bounds, item1)
                 # Lookup for this chunk in cache
@@ -1979,8 +1982,8 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
         else:
             start = 0
         # Lookup for item2
-        if item2 >= b0:
-            if item2 < b1:
+        if nan_aware_ge(item2, b0):
+            if nan_aware_lt(item2, b1):
                 # Search the appropriate chunk in bounds cache
                 nchunk2 = bisect_right(bounds, item2)
                 if nchunk2 != nchunk:
@@ -2012,7 +2015,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
     searchLastRow = previous_api(search_last_row)
 
     def get_chunkmap(self):
-        """Compute a map with the interesting chunks in index"""
+        """Compute a map with the interesting chunks in index."""
 
         if profile:
             tref = time()
@@ -2122,7 +2125,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
     getLookupRange = previous_api(get_lookup_range)
 
     def _f_remove(self, recursive=False):
-        """Remove this Index object"""
+        """Remove this Index object."""
 
         # Index removal is always recursive,
         # no matter what `recursive` says.
@@ -2155,7 +2158,6 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
   slicesize := %s
   blocksize := %s
   superblocksize := %s
-  filters := %s
   dirty := %s
   byteorder := %r""" % (self._v_pathname, cpathname,
                         self.optlevel, self.kind,
@@ -2163,8 +2165,7 @@ class Index(NotLoggedMixin, indexesextension.Index, Group):
                         self.nelements,
                         self.chunksize, self.slicesize,
                         self.blocksize, self.superblocksize,
-                        self.filters, self.dirty,
-                        self.byteorder)
+                        self.dirty, self.byteorder)
         retstr += "\n  sorted := %s" % self.sorted
         retstr += "\n  indices := %s" % self.indices
         retstr += "\n  ranges := %s" % self.ranges
