@@ -173,7 +173,7 @@ class Leaf(Node):
     # `````````````````````````
     @lazyattr
     def filters(self):
-        """Filter properties for this leaf
+        """Filter properties for this leaf.
 
         See Also
         --------
@@ -273,10 +273,8 @@ class Leaf(Node):
         return self.nrows
 
     def __str__(self):
-        """
-        The string representation for this object is its pathname in
-        the HDF5 object tree plus some additional metainfo.
-        """
+        """The string representation for this object is its pathname in the
+        HDF5 object tree plus some additional metainfo."""
 
         # Get this class name
         classname = self.__class__.__name__
@@ -300,6 +298,7 @@ class Leaf(Node):
         """Code to be run after node creation and before creation logging.
 
         This method gets or sets the flavor of the leaf.
+
         """
 
         super(Leaf, self)._g_post_init_hook()
@@ -365,16 +364,23 @@ class Leaf(Node):
         # Compute the nrowsinbuf
         rowsize = self.rowsize
         buffersize = params['IO_BUFFER_SIZE']
-        nrowsinbuf = buffersize // rowsize
+        if rowsize != 0:
+            nrowsinbuf = buffersize // rowsize
+            if self.__class__.__name__ == "Table":
+                # The number of rows in buffer needs to be an exact multiple of
+                # chunkshape[0] for queries using indexed columns.
+                # Fixes #319 and probably #409 too.
+                nrowsinbuf -= nrowsinbuf % self.chunkshape[0]
+        else:
+            nrowsinbuf = 1
 
         # tableextension.pyx performs an assertion
         # to make sure nrowsinbuf is greater than or
         # equal to the chunksize.
         # See gh-206 and gh-238
-        if self.chunkshape is not None:
-            chunksize = numpy.asarray(self.chunkshape).prod()
-            if nrowsinbuf < chunksize:
-                nrowsinbuf = chunksize
+        if self.chunkshape is not None and self.__class__.__name__ == "Table":
+            if nrowsinbuf < self.chunkshape[0]:
+                nrowsinbuf = self.chunkshape[0]
 
         # Safeguard against row sizes being extremely large
         if nrowsinbuf == 0:
@@ -402,23 +408,18 @@ very small/large chunksize, you may want to increase/decrease it."""
 
         if warn_negstep and step and step < 0:
             raise ValueError("slice step cannot be negative")
-        # (start, stop, step) = slice(start, stop, step).indices(nrows)
-        # The next function is a substitute for slice().indices in order to
-        # support full 64-bit integer for slices even in 32-bit machines.
-        # F. Alted 2005-05-08
-        start, stop, step = utilsextension.get_indices(start, stop, step, long(nrows))
-        return (start, stop, step)
+
+        #if start is not None: start = long(start)
+        #if stop is not None: stop = long(stop)
+        #if step is not None: step = long(step)
+
+        return slice(start, stop, step).indices(long(nrows))
 
     _processRange = previous_api(_process_range)
 
-    # This method is appropiate for calls to read() methods
+    # This method is appropriate for calls to read() methods
     def _process_range_read(self, start, stop, step, warn_negstep=True):
         nrows = self.nrows
-        if start is None and stop is None:
-        #if start is None and stop is None and step is None:
-            start = 0
-            stop = nrows
-            #step = 1
         if start is not None and stop is None and step is None:
             # Protection against start greater than available records
             # nrows == 0 is a special case for empty objects
@@ -431,7 +432,7 @@ very small/large chunksize, you may want to increase/decrease it."""
             else:
                 stop = start + 1
         # Finally, get the correct values (over the main dimension)
-        start, stop, step = self._process_range(start, stop, step, 
+        start, stop, step = self._process_range(start, stop, step,
                                                 warn_negstep=warn_negstep)
         return (start, stop, step)
 
@@ -550,7 +551,7 @@ very small/large chunksize, you may want to increase/decrease it."""
             # Get the True coordinates (64-bit indices!)
             coords = numpy.asarray(key.nonzero(), dtype='i8')
             coords = numpy.transpose(coords)
-        elif key.dtype.kind == 'i':
+        elif key.dtype.kind == 'i' or key.dtype.kind == 'u':
             if len(key.shape) > 2:
                 raise IndexError(
                     "Coordinate indexing array has incompatible shape")
@@ -563,6 +564,14 @@ very small/large chunksize, you may want to increase/decrease it."""
             else:
                 # For 1-dimensional datasets
                 coords = numpy.asarray(key, dtype="i8")
+
+            # handle negative indices
+            idx = coords < 0
+            coords[idx] = (coords + self.shape)[idx]
+
+            # bounds check
+            if numpy.any(coords < 0) or numpy.any(coords >= self.shape):
+                raise IndexError("Index out of bounds")
         else:
             raise TypeError("Only integer coordinates allowed.")
         # We absolutely need a contiguous array
@@ -725,10 +734,10 @@ very small/large chunksize, you may want to increase/decrease it."""
     def flush(self):
         """Flush pending data to disk.
 
-        Saves whatever remaining buffered data to disk. It also releases I/O
-        buffers, so if you are filling many datasets in the same PyTables
-        session, please call flush() extensively so as to help PyTables to keep
-        memory requirements low.
+        Saves whatever remaining buffered data to disk. It also releases
+        I/O buffers, so if you are filling many datasets in the same
+        PyTables session, please call flush() extensively so as to help
+        PyTables to keep memory requirements low.
 
         """
 
