@@ -10,7 +10,7 @@
 #
 ########################################################################
 
-"""Here is defined the VLArray class"""
+"""Here is defined the VLArray class."""
 
 import sys
 
@@ -43,7 +43,7 @@ class VLArray(hdf5extension.VLArray, Leaf):
     homogeneous elements, called *atoms*. Like Table datasets (see
     :ref:`TableClassDescr`), variable length arrays can have only one
     dimension, and the elements (atoms) of their rows can be fully
-    multidimensional.  VLArray objects do also support compression.
+    multidimensional.
 
     When reading a range of rows from a VLArray, you will *always* get
     a Python list of objects of the current flavor (each of them for a
@@ -54,6 +54,18 @@ class VLArray(hdf5extension.VLArray, Leaf):
     inherits all the public attributes and methods that Leaf (see
     :ref:`LeafClassDescr`) already provides.
 
+    .. note::
+
+          VLArray objects also support compression although compression
+          is only performed on the data structures used internally by
+          the HDF5 to take references of the location of the variable
+          length data. Data itself (the raw data) are not compressed
+          or filtered.
+
+          Please refer to the `VLTypes Technical Note
+          <http://www.hdfgroup.org/HDF5/doc/TechNotes/VLTypes.html>`_
+          for more details on the topic.
+
     Parameters
     ----------
     parentnode
@@ -117,9 +129,9 @@ class VLArray(hdf5extension.VLArray, Leaf):
         vlarray.append([5, 6, 9, 8])
 
         # Now, read it through an iterator:
-        print '-->', vlarray.title
+        print('-->', vlarray.title)
         for x in vlarray:
-            print '%s[%d]--> %s' % (vlarray.name, vlarray.nrow, x)
+            print('%s[%d]--> %s' % (vlarray.name, vlarray.nrow, x))
 
         # Now, do the same with native Python strings.
         vlarray2 = fileh.create_vlarray(fileh.root, 'vlarray2',
@@ -129,14 +141,14 @@ class VLArray(hdf5extension.VLArray, Leaf):
         vlarray2.flavor = 'python'
 
         # Append some (variable length) rows:
-        print '-->', vlarray2.title
+        print('-->', vlarray2.title)
         vlarray2.append(['5', '66'])
         vlarray2.append(['5', '6', '77'])
         vlarray2.append(['5', '6', '9', '88'])
 
         # Now, read it through an iterator:
         for x in vlarray2:
-            print '%s[%d]--> %s' % (vlarray2.name, vlarray2.nrow, x)
+            print('%s[%d]--> %s' % (vlarray2.name, vlarray2.nrow, x))
 
         # Close the file.
         fileh.close()
@@ -541,7 +553,7 @@ class VLArray(hdf5extension.VLArray, Leaf):
         ::
 
             for row in vlarray.iterrows(step=4):
-                print '%s[%d]--> %s' % (vlarray.name, vlarray.nrow, row)
+                print('%s[%d]--> %s' % (vlarray.name, vlarray.nrow, row))
 
         .. versionchanged:: 3.0
            If the *start* parameter is provided and *stop* is None then the
@@ -585,7 +597,7 @@ class VLArray(hdf5extension.VLArray, Leaf):
         return self
 
     def _init_loop(self):
-        """Initialization for the __iter__ iterator"""
+        """Initialization for the __iter__ iterator."""
 
         self._nrowsread = self._start
         self._startb = self._start
@@ -598,7 +610,8 @@ class VLArray(hdf5extension.VLArray, Leaf):
     def next(self):
         """Get the next element of the array during an iteration.
 
-        The element is returned as a list of objects of the current flavor.
+        The element is returned as a list of objects of the current
+        flavor.
 
         """
 
@@ -697,7 +710,7 @@ class VLArray(hdf5extension.VLArray, Leaf):
                                                               nobjects))
             try:
                 nparr[:] = value
-            except Exception, exc:  # XXX
+            except Exception as exc:  # XXX
                 raise ValueError("Value parameter:\n'%r'\n"
                                  "cannot be converted into an array object "
                                  "compliant vlarray[%s] row: \n'%r'\n"
@@ -809,12 +822,12 @@ class VLArray(hdf5extension.VLArray, Leaf):
         """Read rows specified in `coords`."""
         rows = []
         for coord in coords:
-            rows.append(self.read(coord)[0])
+            rows.append(self.read(long(coord))[0])
         return rows
 
     def _g_copy_with_stats(self, group, name, start, stop, step,
                            title, filters, chunkshape, _log, **kwargs):
-        """Private part of Leaf.copy() for each kind of leaf"""
+        """Private part of Leaf.copy() for each kind of leaf."""
 
         # Build the new VLArray object
         object = VLArray(
